@@ -1,7 +1,7 @@
 """Functions to support the Extended Python Debugger.
-$Id: fns.py,v 1.45 2007/05/23 09:25:43 rockyb Exp $"""
+$Id: fns.py,v 1.52 2008/12/26 01:56:52 rockyb Exp $"""
 # -*- coding: utf-8 -*-
-#   Copyright (C) 2007 Rocky Bernstein
+#   Copyright (C) 2007, 2008 Rocky Bernstein
 #
 #    This program is free software; you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -39,10 +39,14 @@ def arg_split(s,posix=False):
 def checkline(obj, filename, lineno):
     """Check whether specified line seems to be executable.
 
-    Return `lineno` if it is, 0 if not (e.g. a docstring, comment, blank
+    Return LINENO if it is, 0 if not (e.g. a docstring, comment, blank
     line or EOF). Warning: testing is not comprehensive.
     """
-    line = linecache.getline(filename, lineno)
+    if 2 == linecache.getline.func_code.co_argcount:
+        line = linecache.getline(filename, lineno)
+    else:
+        line = linecache.getline(filename, lineno, obj.curframe.f_globals)
+        pass
     if not line:
         obj.errmsg('End of file')
         return 0
@@ -50,7 +54,7 @@ def checkline(obj, filename, lineno):
     # Don't allow setting breakpoint at a blank line
     if (not line or (line[0] == '#') or
          (line[:3] == '"""') or line[:3] == "'''"):
-        obj.errmsg('Blank or comment')
+        obj.errmsg('Blank, doc string, or comment')
         return 0
     return lineno
 
@@ -241,16 +245,16 @@ def is_exec_stmt(frame):
     """Return True if we are looking at an exec statement"""
     return frame.f_back is not None and op_at_frame(frame.f_back)=='EXEC_STMT'
 
-def get_last_tb_or_frame_tb(frameno=1):
+def get_last_tb_or_frame_tb():
 
     """Intended to be used going into post mortem routines.  If
     sys.last_traceback is set, we will return that and assume that
     this is what post-mortem will want. If sys.last_traceback has not
     been set, then perhaps we *about* to raise an error and are
-    fielding an exception. So assume that sys.exec_info()[frameno]
+    fielding an exception. So assume that sys.exc_info()[2]
     is where we want to look."""
 
-    tb = sys.exc_info()[frameno]
+    tb = sys.exc_info()[2]
 
     try:
         if inspect.istraceback(sys.last_traceback):
@@ -260,6 +264,49 @@ def get_last_tb_or_frame_tb(frameno=1):
         pass
     return tb
 
+def get_brkpt_lineno(obj, arg):
+    """get_brkpt_lineno(obj,arg)->(filename, file, lineno)
+    
+    See if arg is a line number or a function name.  Return what
+    we've found. None can be returned as a value in the triple.
+
+    obj should be some sort of Gdb object and contain a curframe,
+    errmsg and lineinfo method.
+    """
+    funcname, filename = (None, None)
+    try:
+        # First see if the breakpont is an integer
+        lineno = int(arg)
+        filename = obj.curframe.f_code.co_filename
+    except ValueError:
+        try:
+            func = eval(arg, obj.curframe.f_globals,
+                        obj.curframe.f_locals)
+        except:
+            func = arg
+        try:
+            # See if agument is a function name
+            if hasattr(func, 'im_func'):
+                func = func.im_func
+            code = func.func_code
+            # use co_name to identify the bkpt (function names
+            #could be aliased, but co_name is invariant)
+            lineno = code.co_firstlineno
+            filename = code.co_filename
+        except:
+            # The last possibility is that a breakpoint argument can be
+            # is some sort of file + linenumber.
+            (ok, filename, ln) = obj.lineinfo(arg)
+            if not ok:
+                obj.errmsg(('The specified object %s is not'
+                            +' a function, or not found'
+                            +' along sys.path or no line given.')
+                           % str(repr(arg)))
+                return (None, None, None)
+            funcname = ok
+            lineno = int(ln)
+    return (funcname, filename, lineno)
+
 def print_dict(s, obj, title):
     if hasattr(obj, "__dict__"):
         d=obj.__dict__
@@ -360,6 +407,78 @@ def print_stack_trace(obj, count=None):
         pass
     return
 
+def runhooks(obj, hook_list, *args):
+    for hook in hook_list:
+        try: 
+            hook(obj, args)
+        except:
+            pass
+        return
+
+def search_python_file(filename, directories=sys.path, 
+                       module_globals=None):
+    fullname = filename
+    try:
+        stat = os.stat(fullname)
+    except os.error, msg:
+        basename = os.path.split(filename)[1]
+
+        # Try for a __loader__, if available
+        if module_globals and '__loader__' in module_globals:
+            name = module_globals.get('__name__')
+            loader = module_globals['__loader__']
+            get_source = getattr(loader, 'get_source', None)
+
+            if name and get_source:
+                if basename.startswith(name.split('.')[-1]+'.'):
+                    try:
+                        data = get_source(name)
+                    except (ImportError, IOError):
+                        pass
+                    else:
+                        if data is None:
+                            # No luck, the PEP302 loader cannot find the source
+                            # for this module.
+                            return []
+                        cache[filename] = (
+                            len(data), None,
+                            [line+'\n' for line in data.splitlines()], fullname
+                        )
+                        return cache[filename][2]
+
+        # Try looking through the module search path, taking care to handle packages.
+
+        if basename == '__init__.py':
+            # filename referes to a package
+            basename = filename
+
+        for dirname in sys.path:
+            # When using imputil, sys.path may contain things other than
+            # strings; ignore them when it happens.
+            try:
+                fullname = os.path.join(dirname, basename)
+            except (TypeError, AttributeError):
+                # Not sufficiently string-like to do anything useful with.
+                pass
+            else:
+                try:
+                    stat = os.stat(fullname)
+                    break
+                except os.error:
+                    pass
+        else:
+            # No luck
+##          print '*** Cannot stat', filename, ':', msg
+            return []
+    try:
+        fp = open(fullname, 'rU')
+        lines = fp.readlines()
+        fp.close()
+    except IOError, msg:
+##      print '*** Cannot open', fullname, ':', msg
+        return []
+
+
 def search_file(filename, directories, cdir):
     """Return a full pathname for filename if we can find one. path
     is a list of directories to prepend to filename. If no file is
@@ -411,49 +530,6 @@ def parse_filepos(obj, arg):
         # no colon: can be lineno or function
         return get_brkpt_lineno(obj, arg)
 
-def get_brkpt_lineno(obj, arg):
-    """get_brkpt_lineno(obj,arg)->(filename, file, lineno)
-    
-    See if arg is a line number or a function name.  Return what
-    we've found. None can be returned as a value in the triple.
-
-    obj should be some sort of Gdb object and contain a curframe,
-    errmsg and lineinfo method.
-    """
-    funcname, filename = (None, None)
-    try:
-        # First see if the breakpont is an integer
-        lineno = int(arg)
-        filename = obj.curframe.f_code.co_filename
-    except ValueError:
-        try:
-            func = eval(arg, obj.curframe.f_globals,
-                        obj.curframe.f_locals)
-        except:
-            func = arg
-        try:
-            # See if agument is a function name
-            if hasattr(func, 'im_func'):
-                func = func.im_func
-            code = func.func_code
-            # use co_name to identify the bkpt (function names
-            #could be aliased, but co_name is invariant)
-            lineno = code.co_firstlineno
-            filename = code.co_filename
-        except:
-            # The last possibility is that a breakpoint argument can be
-            # is some sort of file + linenumber.
-            (ok, filename, ln) = obj.lineinfo(arg)
-            if not ok:
-                obj.errmsg(('The specified object %s is not'
-                            +' a function, or not found'
-                            +' along sys.path or no line given.')
-                           % str(repr(arg)))
-                return (None, None, None)
-            funcname = ok
-            lineno = int(ln)
-    return (funcname, filename, lineno)
-
 def whence_file(py_script):
     """Do a shell-like path lookup for py_script and return the results.
     If we can't find anything return py_script"""
