@@ -1,6 +1,6 @@
 """Handles gdb-like command processing.
 (See also pydb.doc for documentation.)
-$Id: gdb.py.in,v 1.151 2008/05/21 11:02:51 rockyb Exp $"""
+$Id: gdb.py.in,v 1.159 2008/12/26 04:32:19 rockyb Exp $"""
 # -*- coding: utf-8 -*-
 #   Copyright (C) 2007, 2008 Rocky Bernstein (rocky@gnu.org)
 #
@@ -22,7 +22,7 @@ $Id: gdb.py.in,v 1.151 2008/05/21 11:02:51 rockyb Exp $"""
 _debugger_name = 'pydb'
 
 # The name of the debugger we are currently going by.
-_version = '1.23'
+_version = '1.25'
 
 import atexit, inspect, linecache, os, pprint, pydoc, shlex
 import bdb, disassemble, re, subcmd, sys, types
@@ -1242,11 +1242,17 @@ is no Python file."""
                    self.mainpyfile = ""
         else:
             # XXX how to clean up name space ??
-            self.mainpyfile=fname
-            self._program_sys_argv = [self.mainpyfile]
-            self._runscript(self.mainpyfile)
-            # Fake up curframe? 
-            return self.quitting
+            if os.path.exists(fname):
+                self.mainpyfile=fname
+                self._program_sys_argv = [self.mainpyfile]
+                self._runscript(self.mainpyfile)
+                # Fake up curframe? 
+                return self.quitting
+            else:
+                self.errmsg("file '%s' does not exist" % fname)
+                return None
+            pass
+        pass
 
     def do_finish(self, arg):
         """finish
@@ -1264,12 +1270,10 @@ is no Python file."""
         return True
 
     def do_frame(self, arg):
-        """frame frame-number
-        Move the current frame to the specified frame number.
+        """frame [frame-number]
 
-        If using gdb dialect up matches the gdb: 0 is the most recent
-        frame.  Otherwise we match Python's stack: 0 is the oldest
-        frame.
+        Move the current frame to frame `frame-number' if specified,
+        or the current frame, 0 if no frame number specified.
 
         A negative number indicates position from the other end.
         So 'frame -1' moves when gdb dialect is in effect moves
@@ -1278,6 +1282,7 @@ is no Python file."""
         if not self.stack:
             self.msg("Program has no stack frame set.")
             return False
+        arg = arg.strip() or '0'
         arg = self.get_an_int(arg,
                               ("The 'frame' command requires a frame number."+
                                " Got: %s") % arg)
@@ -1481,10 +1486,18 @@ we are in interactive mode, we'll prompt to make sure.
         if filename is None: return
         breaklist = self.get_file_breaks(filename)
 
+        # Python 2.5 or greater has 3 arg getline which handles
+        # eggs and zip files
+        if 3 == linecache.getline.func_code.co_argcount:
+            getline = lambda f, l: linecache.getline(f, l, 
+                                                     self.curframe.f_globals)
+        else:
+            getline = lambda f, l: linecache.getline(f, l)
+
         # We now have range information. Do the listing.
         try:
             for lineno in range(first, last+1):
-                line = linecache.getline(filename, lineno)
+                line = getline(filename, lineno)
                 if not line:
                     self.msg('[EOF]')
                     break
@@ -1508,7 +1521,7 @@ we are in interactive mode, we'll prompt to make sure.
         """n(ext) [count]
 Continue execution until the next line in the current function is reached or it returns.
 
-With an integer argument, perform that many times."""
+With an integer argument, perform 'next' that many times."""
         if not self.is_running(): return
         try:
             # 0 means stop now or step 1, so we subtract 1.
@@ -1553,7 +1566,7 @@ With an integer argument, perform that many times."""
                     return False
                 
             self.msg(pprint.pformat(val))
-        except EOFError:
+        except:
             pass
 
     def do_pwd(self, arg):
@@ -1593,7 +1606,7 @@ Run python as a command subshell.
 
         # Python does it's own history thing.
         # Make sure it doesn't damage ours.
-        if self.readline:
+        if hasattr(self, 'readline') and self.readline:
             try:
                 self.write_history_file
             except IOError:
@@ -1610,7 +1623,7 @@ Run python as a command subshell.
             interact(banner='Pydb python shell')
 
         # Restore our history if we can do so.
-        if self.readline and self.histfile is not None:
+        if hasattr(self, 'readline') and self.readline and self.histfile is not None:
             try:
                 self.readline.read_history_file(self.histfile)
             except IOError:
@@ -1866,15 +1879,22 @@ Send a signal to the debugged process.
         return False
         
     def do_source(self, arg):
-        """source FILE
+        """source [-v] FILE
         Read debugger commands from a file named FILE.
+        Optional -v switch (before the filename) causes each command in
+        FILE to be echoed as it is executed.
         Note that the file '.pydbrc' is read automatically
         in this way when pydb is started.
 
         An error in any command terminates execution of the command
         file and control is returned to the console."""
+        args = arg.split()
+        verbose=False
+        if len(args) == 2 and args[0] == '-v': 
+            arg=args[1]
+            verbose=True
         self.setup_source(os.path.expanduser(arg), True);
-        rc = self.execRcLines()
+        rc = self.execRcLines(verbose)
         if rc == 1:  return True
         return False
 
