@@ -5,16 +5,16 @@ or runevel.
 
 (See also pydb.doc for documentation.)
 
-$Id: pydb.py.in,v 1.155 2008/05/21 11:02:51 rockyb Exp $"""
+$Id: pydb.py.in,v 1.161 2008/12/22 15:17:25 rockyb Exp $"""
 
 # This part contains user entry subroutines.
 
 __title__ = 'pydb'
 
 # The name of the debugger we are currently going by.
-__version__ = '1.23'
+__version__ = '1.25'
 
-import inspect, os, re, sys, traceback
+import inspect, os, re, sys, traceback, types
 
 import fns
 
@@ -224,6 +224,9 @@ class Pdb(Gdb):
 
     See the Gdb class documentation for more information and information on
     the meanings of the parameters.
+
+    If there is something you want to run before/after on each stopping
+    point, set precmd_hook and/or postcmd_hook
     """
 
     def __init__(self, completekey='tab', stdin=None, stdout=None,
@@ -238,8 +241,33 @@ class Pdb(Gdb):
                                                "n", "next", "return", "jump"])
         self.__show_annotations_postcmd = set(["down", "frame", "up"])
         self.__show_annotations = True
+        self.precmd_hooks = []
+        self.preloop_hooks = []
+        self.postcmd_hooks = []
+        self.postloop_hooks = []
         return
 
+    def __annotation(self, label, cmd, *args):
+        self.msg(chr(26) + chr(26) + label) # chr(26) is Ctrl-Z
+        cmd(*args)
+        self.msg(chr(26) + chr(26))
+
+
+    def precmd(self, line):
+        fns.runhooks(self, self.precmd_hooks, line)
+        return line
+        
+    def preloop(self):
+        fns.runhooks(self, self.preloop_hooks)
+        if self.annotate > 0:
+            if self.__show_annotations:
+                # if we are here, the stack frames have changed outside the
+                # command loop (e.g. after a "continue" command), so we show
+                # the annotations again
+                self.__annotation('breakpoints', self.do_info, 'breakpoints')
+                self.__annotation('stack', self.do_where, 0)
+                self.__annotation('locals', self.do_info, 'locals')
+
     def postcmd(self, stop, line):
         if self.annotate > 0:
             cmd = ""
@@ -251,22 +279,12 @@ class Pdb(Gdb):
             if cmd in self.__show_annotations_postcmd:
                 self.__annotation('stack', self.do_where, 0)
                 self.__annotation('locals', self.do_info, 'locals')
+        fns.runhooks(self, self.postcmd_hooks, stop, line)
         return stop
 
-    def preloop(self):
-        if self.annotate > 0:
-            if self.__show_annotations:
-                # if we are here, the stack frames have changed outside the
-                # command loop (e.g. after a "continue" command), so we show
-                # the annotations again
-                self.__annotation('breakpoints', self.do_info, 'breakpoints')
-                self.__annotation('stack', self.do_where, 0)
-                self.__annotation('locals', self.do_info, 'locals')
-
-    def __annotation(self, label, cmd, *args):
-        self.msg(chr(26) + chr(26) + label) # chr(26) is Ctrl-Z
-        cmd(*args)
-        self.msg(chr(26) + chr(26))
+    def postloop(self):
+        fns.runhooks(self, self.postloop_hooks)
+        return
 
 # -- end class Pdb
 
@@ -274,73 +292,93 @@ class Pdb(Gdb):
 # Other top-level routines
 #########################################################
 
+# Start here: note status is new and params have changed...
 def debugger(dbg_cmds=None, add_exception_hook=True, add_threaddbg=False,
-             new_instance=False):
-    """Enter the debugger at the calling stack frame.  This is useful to
-    hard-code a breakpoint at a given point in a program, even if the code
-    is not otherwise being debugged (e.g., when an assertion fails).
-    Leaving this the debugger terminates the program.
+             status='start'):
+    """Enter the debugger at the calling stack frame.  This can be
+    used as a way to hard-code a breakpoint in a program, even if the
+    code is not otherwise being debugged (e.g., when an assertion
+    fails).  Leaving the debugger, i.e. issuing a "quit" command,
+    terminates the program.
 
-    dbg_cmds is an optional list of debugger commands you want to run.
+    dbg_cmds, a list of strings, is an optional list of debugger
+    commands you want to run.
 
     If add_exception is True (the default), we will install an exception hook
     to call the post-mortem debugger on an unhandled exception.
 
     If the debugger was called previously, normally we use that. This
-    way any previous debugger state that was set gets used. If you
-    don't the debugger state to be used, set parameter new_instance
-    True.
+    way any previous debugger state that was set gets used.
+
+    Setting the parameter `status' to the string 'new' will cause a
+    new instance to be created which might wiping out other
+    instances which might contain information like breakpoints or
+    debugger settings.
+    
+    If instead `status' is set to 'continue', an existing instance
+    will resume. Omitting status or setting it to 'start' will start
+    the debugger which might be a 'new' depending on whether or not
+    an instance has been created. 
+
+    In all cases a string indicating what was done is returned.
+
+    Since debugging slows down a program, one may want to stop and
+    continue it at various points in your program.
     """
     global _pydb_trace
+
+    if types.BooleanType == type(status):
+        if status: status = 'new' 
+        else: status = 'start'
+        print "debugger() interface has changed see docstring."
+
     try:
         if not isinstance(_pydb_trace, Pdb):
             print "Your program should not use _pydb_trace"
-            return
+            return '_pydb_trace conflict'
     except NameError:
-        new_instance = True
+        status = 'new'
     except:
         print "Unknown error"
-        return
+        return 'Unknown error'
 
-    if new_instance:
+    back_frame = inspect.currentframe().f_back
+    if 'new' == status:
         if add_threaddbg:
             import threaddbg
             _pydb_trace = threaddbg.threadDbg()
         else:
             _pydb_trace = Pdb()
-        _pydb_trace.reset()
         _pydb_trace._program_sys_argv = list(sys.argv)
         _pydb_trace._sys_argv = list(_pydb_trace._program_sys_argv)
         _pydb_trace._sys_argv[:0] = [__title__]
         _pydb_trace.main_dirname = os.path.dirname(sys.argv[0])
 
+    _pydb_trace.curframe  = back_frame
+    _pydb_trace.stopframe = back_frame
+    if 'continue' == status:
+        pass
+    elif _pydb_trace.running:
+        return 'running'
+    _pydb_trace.running = True
+
     # We don't support "run" so we'll make "run" and "R" be "restart"
     _pydb_trace.do_R = _pydb_trace.do_run = _pydb_trace.do_restart
-    _pydb_trace.curframe=inspect.currentframe()
+
+    _pydb_trace.curframe  = back_frame
+    _pydb_trace.stopframe = back_frame
 
     # Dispatcher needs to field BdbQuit exception
     _pydb_trace.field_BdbQuit   = True
 
-    if _pydb_trace.running:
-        _pydb_trace.do_next('2')
-        return
-    _pydb_trace.running = True
-
     if dbg_cmds != None:
         _pydb_trace.cmdqueue = list(dbg_cmds)
 
     if add_exception_hook:
         sys.excepthook = exception_hook
 
-    # Set to skip over return statement
-    _pydb_trace.step_ignore = 1
-    # There can be no other commands after the following one
-    # unless we adjust _pydb_trace.step_ignore accordingly.
-    # Note in some cases we stop in the "return" below and in some
-    # cases we step one more so we need to be conservitive in the
-    # step count. I don't know why weird behavior.
     _pydb_trace.set_trace(_pydb_trace.curframe)
-    return
+    return status
 
 def exception_hook(type, value, tb, dbg_cmds=None, cmdfile=None):
     """An exception hook to call pydb's post-mortem debugger.
@@ -363,7 +401,7 @@ def help():
     doc_file='%s.doc' % __title__
     for dirname in sys.path:
         fullname = os.path.join(dirname, __title__, doc_file)
-        if os.path.exists(fullname):
+        if os.path.isfile(fullname):
             sts = os.system('${PAGER-more} '+fullname)
             if sts: print 'Pager exit status: %s' % str(sts)
             break
@@ -394,7 +432,7 @@ def help():
 def pm(dbg_cmds=None, frameno=1, p=None):
 
     """Set up post-mortem debugging using the last traceback.  But if
-    there is no traceback, we'll assume that sys.exec_info() contains
+    there is no traceback, we'll assume that sys.exc_info() contains
     what we want and frameno is the index location of where we want
     to start.
 
@@ -402,9 +440,7 @@ def pm(dbg_cmds=None, frameno=1, p=None):
     'p', is an optional pydb.Pdb object.
     """
 
-    # frameno+1 because we are about to add one more level of call
-    # in get_last_tb_or_frame_tb
-    tb = fns.get_last_tb_or_frame_tb(frameno+1)
+    tb = fns.get_last_tb_or_frame_tb()
     post_mortem(tb, dbg_cmds=dbg_cmds, p=p)
     return
 
@@ -412,7 +448,7 @@ def post_mortem(t=None, dbg_cmds=None, cmdfile=None, frameno=1, p=None):
     """Enter debugger read loop after your program has crashed.
 
     If no traceback parameter, t, is supplied, the last traceback and
-    if that doesn't exist either we'll assume that sys.exec_info()
+    if that doesn't exist either we'll assume that sys.exc_info()
     contains what we want and frameno is the index location of where
     we want to start.
 
@@ -436,7 +472,7 @@ def post_mortem(t=None, dbg_cmds=None, cmdfile=None, frameno=1, p=None):
     if t is None:
         # frameno+1 because we are about to add one more level of call
         # in get_last_tb_or_frame_tb
-        t = fns.get_last_tb_or_frame_tb(frameno+1)
+        t = fns.get_last_tb_or_frame_tb()
         if t is None:
             print "Can't find traceback for post_mortem " + \
                   "in sys.last_traceback or sys.exec_info()"
@@ -499,6 +535,15 @@ def post_mortem(t=None, dbg_cmds=None, cmdfile=None, frameno=1, p=None):
         pass
     return
 
+def uncaught_exception(p):
+    traceback.print_exc()
+    print "Uncaught exception. Entering post mortem debugging"
+    t = sys.exc_info()[2]
+    p.interaction(t.tb_frame,t)
+    print "Post mortem debugger finished."
+    return None
+
+
 def run(statement, globals=None, locals=None):
 
     """Execute the statement (given as a string) under debugger
@@ -529,7 +574,10 @@ def runeval(expression, globals=None, locals=None):
 
     p = Pdb()
     p.running=True
-    return p.runeval(expression, globals, locals)
+    try:
+        return p.runeval(expression, globals, locals)
+    except:
+        uncaught_exception(p)
 
 def runcall(*args, **kwds):
 
@@ -543,7 +591,10 @@ def runcall(*args, **kwds):
 
     p = Pdb()
     p.running=True
-    return p.runcall(*args, **kwds)
+    try:
+        return p.runcall(*args, **kwds)
+    except:
+        uncaught_exception(p)
 
 def runl(*args):
 
@@ -676,7 +727,7 @@ def main(p=None):
         p._program_sys_argv = list(sys.argv)
 
         mainpyfile = p._program_sys_argv[0] # Get script filename.
-        if not os.path.exists(mainpyfile):
+        if not os.path.isfile(mainpyfile):
             mainpyfile=fns.whence_file(mainpyfile)
             if not os.path.exists(mainpyfile):
                 print "pydb: Python script file '%s' does not exist" \
