@@ -1,4 +1,4 @@
-"""$Id: pydbbdb.py,v 1.44 2008/05/17 10:08:33 rockyb Exp $
+"""$Id: pydbbdb.py,v 1.50 2008/12/21 10:54:21 rockyb Exp $
 Routines here have to do with the subclassing of bdb.  Defines Python
 debugger Basic Debugger (Bdb) class.  This file could/should probably
 get merged into bdb.py
@@ -28,7 +28,12 @@ class Bdb(bdb.Bdb):
         self._repr.maxset    = 10
         self._repr.maxfrozen = 10
         self._repr.array     = 10
-        self._saferepr = self._repr.repr
+        self._saferepr       = self._repr.repr
+
+        # Certain debugger commands are and actions only applicable
+        # for running programs. Thus we record the running status of
+        # the debugged code.
+        self.running         = False
 
         # A 0 value means stop on this occurrence. A positive value means to
         # skip that many more step/next's.
@@ -41,8 +46,17 @@ class Bdb(bdb.Bdb):
     def __print_call_params(self, frame):
         "Show call paramaters and values"
         self.setup(frame)
-        self.msg(self.format_stack_entry(self.stack[-1],
-                                         include_location=False))
+
+        # Does format_stack_entry() have an 'include_location' parameter?
+        fse_code = self.format_stack_entry.func_code
+        fse_args = fse_code.co_varnames
+        if 'include_location' in fse_args:
+            self.msg(self.format_stack_entry(self.stack[-1],
+                                             include_location=False))
+        else:
+            self.msg(self.format_stack_entry(self.stack[-1]))
+            return
+        return
 
     def __print_location_if_trace(self, frame, include_fntrace=True):
         if self.linetrace or (self.fntrace and include_fntrace):
@@ -51,6 +65,9 @@ class Bdb(bdb.Bdb):
             self.display.displayAny(self.curframe)
             if self.linetrace_delay:
                 time.sleep(self.linetrace_delay)
+                return
+            return
+        return
 
     def bp_commands(self, frame):
 
@@ -129,11 +146,14 @@ class Bdb(bdb.Bdb):
             else: ss = ''
             self.msg('\tbreakpoint already hit %d time%s' %
                      (bp.hits, ss), out)
+        return
 
     def output_break_commands(self):
-        "Output a list of 'break' commands"
-        # FIXME: for now we ae going to assume no breakpoints set
-        # previously
+        """Return a list of 'break' commands. This could be used to save and
+        restore breakpoint status across a restart"""
+        # FIXME: We are going to assume no breakpoints set
+        # previously. bp_no is renumbered breakpoint numbers used
+        # in subsequent "disable" commands.
         bp_no = 0 
         out = []
         for bp in bdb.Breakpoint.bpbynumber:
@@ -178,8 +198,7 @@ class Bdb(bdb.Bdb):
                 #### ARG. All for the below name change.
                 self.do_delete(str(bp.number))
             return True
-        else:
-            return False
+        return False
 
     def canonic(self, filename):
 
@@ -207,6 +226,7 @@ class Bdb(bdb.Bdb):
                 if not canonic: canonic = filename
             canonic = os.path.normcase(canonic)
             self.fncache[filename] = canonic
+            return canonic
         return canonic
 
     def canonic_filename(self, frame):
@@ -232,6 +252,7 @@ class Bdb(bdb.Bdb):
             self.breaks[filename].remove(lineno)
         if not self.breaks[filename]:
             del self.breaks[filename]
+            return brkpts
         return brkpts
 
     def complete(self, text, state):
@@ -309,6 +330,7 @@ class Bdb(bdb.Bdb):
 
             if add_quotes_around_file: filename = "'%s'" % filename
             s += " %s at line %r" % (filename, lineno)
+            return s
         return s
 
     # The following two methods can be called by clients to use
@@ -339,10 +361,12 @@ class Bdb(bdb.Bdb):
             self.quitting = 1
             self.running = False
             sys.settrace(None)
+        return
 
     def reset(self):
         bdb.Bdb.reset(self)
         self.forget()
+        return
 
     def set_trace(self, frame=None):
         """Wrapper to accomodate different versions of Python"""
@@ -353,6 +377,7 @@ class Bdb(bdb.Bdb):
         else:
             # older versions
             self.bdb_set_trace(self)
+        return
 
     def user_call(self, frame, argument_list):
         """This method is called when there is the remote possibility
@@ -373,6 +398,8 @@ class Bdb(bdb.Bdb):
                 self.__print_location_if_trace(frame)
                 if not self.break_here(frame): return
             self.interaction(frame, None)
+            return
+        return
 
     def user_exception(self, frame, (exc_type, exc_value, exc_traceback)):
         """This function is called if an exception occurs,
@@ -389,6 +416,7 @@ class Bdb(bdb.Bdb):
         self.msg("%s:%s" % (str(exc_type_name),
                             str(self._saferepr(exc_value))))
         self.interaction(frame, exc_traceback)
+        return
 
     def user_line(self, frame):
         """This function is called when we stop or break at this line.
@@ -406,9 +434,18 @@ class Bdb(bdb.Bdb):
             # Don't stop if we are looking at a def for which a breakpoint
             # has not been set.
             filename = self.filename(self.canonic_filename(frame))
-            line = linecache.getline(filename, inspect.getlineno(frame))
-            # No don't have a breakpoint. So we are either
-            # stepping or here be of line tracing.
+
+            # Python 2.5 or greater has 3 arg getline which handles
+            # eggs and zip files
+            if 3 == linecache.getline.func_code.co_argcount:
+                line = linecache.getline(filename, inspect.getlineno(frame),
+                                         frame.f_globals)
+            else:
+                line = linecache.getline(filename, inspect.getlineno(frame))
+                pass
+
+            # No, we don't have a breakpoint. So we are either
+            # stepping or here because of line tracing.
             if self.step_ignore > 0:
                 # Don't stop this time, just note a step was done in
                 # step count
@@ -434,6 +471,8 @@ class Bdb(bdb.Bdb):
                 return
         if self.bp_commands(frame):
             self.interaction(frame, None)
+            return
+        return
 
     def user_return(self, frame, return_value):
         """This function is called when a return trap is set here."""
@@ -451,4 +490,6 @@ class Bdb(bdb.Bdb):
         self.__print_location_if_trace(frame, False)
         if self.returnframe != None:
             self.interaction(frame, None)
+            return
+        return
 
