@@ -6,7 +6,7 @@ not always) they are not specific to pydb. They are sort of more
 oriented towards any gdb-like debugger. Also routines that need to be
 changed from cmd are here.
 
-$Id: pydbcmd.py,v 1.47 2008/04/16 01:20:47 rockyb Exp $"""
+$Id: pydbcmd.py,v 1.51 2008/12/26 01:56:52 rockyb Exp $"""
 
 import cmd, linecache, sys, types
 from fns import *
@@ -71,7 +71,7 @@ class Cmd(cmd.Cmd):
 
         statement = 'execfile( "%s")' % filename
         self.running = True
-        self.run(statement, globals=globals_, locals=locals_)
+        self.run(statement)
         return
 
     def default(self, line):
@@ -96,7 +96,16 @@ class Cmd(cmd.Cmd):
             global_vars = None
         try:
             code = compile(line + '\n', '"%s"' % line, 'single')
-            exec code in global_vars, local_vars
+            save_stdout = sys.stdout
+            save_stdin = sys.stdin
+            try:
+                sys.stdin = self.stdin
+                sys.stdout = self.stdout
+                exec code in global_vars, local_vars
+            finally:
+                sys.stdout = save_stdout
+                sys.stdin = save_stdin
+                pass
         except:
             t, v = sys.exc_info()[:2]
             if type(t) == types.StringType:
@@ -198,7 +207,7 @@ See also 'examine' an 'whatis'.
     do_h = do_help
 
     # Can be executed earlier than 'setup' if desired
-    def execRcLines(self):
+    def execRcLines(self, verbose=False):
 
         """Some commands were batched in self.rcLines.  Run as many of
         them as we can now.
@@ -215,6 +224,7 @@ See also 'examine' an 'whatis'.
             for line in rcLines:
                 self.rcLines = self.rcLines[1:]
                 line = line[:-1]
+                if verbose: self.msg('+' + line)
                 if len(line) > 0:
                     # Some commands like step, continue,
                     # return return 1 to indicate execution
@@ -222,8 +232,7 @@ See also 'examine' an 'whatis'.
                     # value is kind of sucky but at present
                     # it's too much work to fix all of the
                     # places needed. So live with it.
-                    if self.onecmd(line) == 1:
-                        return 1
+                    if self.onecmd(line) == 1: return 1
 
     def get_an_int(self, arg, errmsg=None, min_value=None, max_value=None):
         """Another get_int() routine, this one simpler and less stylized
@@ -442,7 +451,17 @@ See also 'examine' an 'whatis'.
 
             if print_line:
                 self.msg_nocr('+ ')
-            line=linecache.getline(filename, lineno)
+                pass
+            if '__loader__' in self.curframe.f_globals:
+                l = self.curframe.f_globals['__loader__']
+                print l
+                pass
+            if 2 == linecache.getline.func_code.co_argcount:
+                line = linecache.getline(filename, lineno)
+            else:
+                line = linecache.getline(filename, lineno, 
+                                         self.curframe.f_globals)
+                pass
             if line and len(line.strip()) != 0:
                 self.print_source_line(lineno, line)
 
